<?php
//************/
// Mobile version to standard browser version switch
//************/
function eb_mobile_available( $content ) {
	$nonce_url = wp_nonce_url(
		trailingslashit( get_bloginfo( 'home' ) ) . '?eb_mobile_action=show_mobile',
		'eb_mobile_nonce_action' 
	);
	$content   .= '<p id="eb-mobile-link"><a href="' . esc_url( $nonce_url ) . '">' . esc_html__(
		'Return to the Mobile Version',
		'eventbrite' 
	) . '</a></p>';

	//all outout properly escaped
	echo $content;//phpcs:ignore
}

function get_eb_mobile_exit() {
	$nonce_url = wp_nonce_url(
		trailingslashit( get_bloginfo( 'home' ) ) . '?eb_mobile_action=reject_mobile',
		'eb_mobile_nonce_action' 
	);

	return '<div class="eb-mobile-exit" id="eb-mobile-exit-link"><a href="' . esc_url( $nonce_url ) . '"> ' . esc_html__(
		'Desktop Version',
		'eventbrite' 
	) . '</a></div>';
}

function eb_mobile_exit() {
	//Output properly escaped already.
	echo get_eb_mobile_exit();//phpcs:ignore
}

function eb_mobile_link() {
	if ( defined( 'WPCACHEHOME' ) ) {
		return;
	}
	$nonce_url = wp_nonce_url(
		trailingslashit( get_bloginfo( 'home' ) ) . '?eb_mobile_action=show_mobile',
		'eb_mobile_nonce_action' 
	);
	echo '<a href="' . esc_url( $nonce_url ) . '">Mobile Version</a>';
}

function eb_mobile_desktop_redirect() {
	echo '
<script type="text/javascript" src="' . esc_url( trailingslashit( get_bloginfo( 'home' ) ) ) . 'wp-content/plugins/eventbrite-mobile/js/user-agent-detect.js"></script>
<script type="text/javascript">
<!--
eb_desktopClientRedirect();
-->
</script>
';
}

function eb_checkForShortcode( $content = '' ) {
	$eb_shortcodes = [
		'eb_listview',
		'eb_hero_listview',
		'eb_calendar',
		'eb_event_details',
		'eb_carousel',
		'eb_upcoming',
		'eb_searchform',
		'eb_emailform',
		'tf_calendar',
		'tf_event_details',
		'tf_carousel',
		'tf_upcoming',
		'tf_justannounced',
		'tf_onsale',
		'tf_hero_listview',
		'tf_listview',
		'tf_searchform',
		'tf_emailform',
		'tf_venuelist',
		'tf_venueinfo',
		'tf_attraction_details',
	];
	foreach ( $eb_shortcodes as $shortcode ) {
		if ( ( false !== strpos( $content, '[' . $shortcode . ' ' ) ) ) {
			return true;
		}
		if ( ( false !== strpos( $content, '[' . $shortcode . ']' ) ) ) {
			return true;
		}
	}

	return false;
}

function eb_checkVenueEnabled() {
	global $eventbrite_music;
	$settings = $eventbrite_music->get_options();
	if (!isset( $settings[ "enable_venue_name" ] ) ) {
		return false;
	}
	return '1' === $settings[ "enable_venue_name" ];
}

if ( ! function_exists( 'eb_add_slug_body_class' ) ) {
	function eb_add_slug_body_class( $classes ) {
		global $post, $wp_query;

		global $eventbrite_mobile;
		$eb_mobileOptions = $eventbrite_mobile->get_options();

		if ( isset( $eb_mobileOptions[ "theme_color" ] ) && ( '' !== $eb_mobileOptions[ "theme_color" ] ) ) {
			$classes[] = $eb_mobileOptions[ "theme_color" ] . '-theme';
		}

		if ( isset( $post ) ) {
			$classes[] = $post->post_type . '-' . $post->post_name;
		}

		if ( isset( $post->eb_events_page_role ) && ! empty( $post->eb_events_page_role ) ) {
			$classes[] = 'eb-event-page';
			$classes[] = 'eb-event-' . $post->eb_events_page_role;
		} else {
			$classes[] = 'non-event-page';
		}

		if ( isset( $wp_query->is_eb_home ) ) {
			$classes[] = 'home';
		}

		if ( isset( $eventbrite_mobile->device ) && ( false !== $eventbrite_mobile->device ) ) {
			$classes[] = 'mobile';
			$classes[] = $eventbrite_mobile->device;
		}

		$classes[] = 'site-id-' . (string) get_current_blog_id();

		if ( isset( $post ) && ( 'page' === get_post_type( $post ) ) ) {
			if ( isset( $post->post_content ) && ( false !== eb_checkForShortcode( $post->post_content ) ) ) {
				$classes[] = 'eventbrite-content';
			} else {
				$classes[] = 'non-event-content';
			}
		}

		return $classes;
	}
}


function tf_getExternalLinks( $event ) {
	if ( isset( $event->externalTicketingUrls ) && is_array( $event->externalTicketingUrls ) ) {
		$extLinkMarkup = '';
		foreach ( $event->externalTicketingUrls as $externalTicketLink ) {
			$extLinkMarkup .= '<span class="ticket-link external-links"><a class="tickets" href="' . $externalTicketLink->url . '" target="_blank">' . $externalTicketLink->name . '</a></span>';
		}

		return $extLinkMarkup;
	}

	return false;
}

if ( ! function_exists( 'tfly_getFbURL' ) ) {
	function tfly_getFbURL( $url = '' ) {
		if ( '' === $url ) {
			return false;
		}

		$urlFB     = $url . '?utm_source=fb1&amp;utm_medium=shr';
		$tinyurlFB = ! is_page( 'event' ) ? tfly_getBitlyUrl( $urlFB ) : '';
		$tinyurlFB = ( $tinyurlFB ) ? $tinyurlFB : $urlFB;

		return $tinyurlFB;
	}
}

if ( ! function_exists( 'tfly_getTweetURL' ) ) {
	function tfly_getTweetURL( $url = '', $event = false ) {
		if ( '' === $url || false === $event ) {
			return false;
		}

		$urlTwitter     = $url . '?utm_source=tw1&amp;utm_medium=shr';
		$tinyurlTwitter = ! is_page( 'event' ) ? tfly_getBitlyUrl( $urlTwitter ) : '';
		$tinyurlTwitter = ( $tinyurlTwitter ) ? $tinyurlTwitter : $urlTwitter;

		if ( $event->headliners[0]->twitterScreenName ) {
			$twitterHeadliner = '@' . $event->headliners[0]->twitterScreenName;
		} else {
			$twitterHeadliner = $event->headliners[0]->name;
		}
		if ( $event->venue->urlTwitter ) {
			if ( trim( wp_parse_url( $event->venue->urlTwitter, PHP_URL_PATH ), '/' ) ) {
				$twitterVenue = '@' . trim( wp_parse_url( $event->venue->urlTwitter, PHP_URL_PATH ), '/' );
			} elseif ( trim( wp_parse_url( $event->venue->urlTwitter, PHP_URL_FRAGMENT ), '!/' ) ) {
				$twitterVenue = '@' . trim( wp_parse_url( $event->venue->urlTwitter, PHP_URL_FRAGMENT ), '!/' );
			} else {
				$twitterVenue = $event->venue->name;
			}
		} else {
			$twitterVenue = $event->venue->name;
		}

		$tweet_desc = 'https://twitter.com/intent/tweet?status=' . rawurlencode( sprintf(
			__(
				"Check it out! %s at %s on %s: %s",
				"eventbrite" 
			),
			$twitterHeadliner,
			$twitterVenue,
			date( "n/j/y", strtotime( $event->startDate ) ),
			$tinyurlTwitter 
		) );

		return $tweet_desc;
	}
}

function headliner_name_for_event( $event ) {
	return $event->name->text;
}

function eb_addArtistMoreToggle() {
	echo '<script type="text/javascript">jQuery(document).on("ready", function (event) { adjustEventInfo(); })</script>';
}
