<?php
/**
 * Sage includes
 *
 * The $sage_includes array determines the code library included in your theme.
 * Add or remove files to the array as needed. Supports child theme overrides.
 *
 * Please note that missing files will produce a fatal error.
 *
 * @link https://github.com/roots/sage/pull/1042
 */
$sage_includes = [
	'lib/utils.php',                 // Utility functions
	'lib/init.php',                  // Initial theme setup and constants
	'lib/wrapper.php',               // Theme wrapper class
	'lib/conditional-tag-check.php', // ConditionalTagCheck class
	'lib/config.php',                // Configuration
	'lib/assets.php',                // Scripts and stylesheets
	'lib/titles.php',                // Page titles
	'lib/extras.php',                // Custom functions
];

foreach ( $sage_includes as $file ) {
	$filepath = locate_template( $file );
	if ( ! $filepath ) {
		wp_die( sprintf( esc_html__( 'Error locating %s for inclusion', 'sage' ), esc_html( $file ) ) );
		continue;
	}

	require_once $filepath;
}
unset( $file, $filepath );

/* This is for compatibility of WP < v4.1 */
if ( ! function_exists( '_navigation_markup' ) ) {
	function _navigation_markup( $links, $class = 'posts-navigation', $screen_reader_text = '' ) {
		if ( empty( $screen_reader_text ) ) {
			$screen_reader_text = __( 'Posts navigation' );
		}

		$template = '
    <nav class="navigation %1$s" role="navigation">
        <h2 class="screen-reader-text">%2$s</h2>
        <div class="nav-links">%3$s</div>
    </nav>';

		return sprintf( $template, sanitize_html_class( $class ), esc_html( $screen_reader_text ), $links );
	}
}

if ( ! function_exists( 'get_the_posts_navigation' ) ) {
	function get_the_posts_navigation( $args = [] ) {
		$navigation = '';

		// Don't print empty markup if there's only one page.
		if ( $GLOBALS[ 'wp_query' ]->max_num_pages > 1 ) {
			$args = wp_parse_args(
				$args,
				[
					'prev_text'          => __( 'Older posts' ),
					'next_text'          => __( 'Newer posts' ),
					'screen_reader_text' => __( 'Posts navigation' ),
				]
			);

			$next_link = get_previous_posts_link( $args[ 'next_text' ] );
			$prev_link = get_next_posts_link( $args[ 'prev_text' ] );

			if ( $prev_link ) {
				$navigation .= '<div class="nav-previous">' . $prev_link . '</div>';
			}

			if ( $next_link ) {
				$navigation .= '<div class="nav-next">' . $next_link . '</div>';
			}

			$navigation = _navigation_markup( $navigation, 'posts-navigation', $args[ 'screen_reader_text' ] );
		}

		return $navigation;
	}
}

if ( ! function_exists( 'the_posts_navigation' ) ) {
	function the_posts_navigation( $args = [] ) {
		echo wp_kses_post(get_the_posts_navigation( $args )); //phpcs:ignore
	}
}

if ( ! function_exists( 'get_the_archive_title' ) ) {
	function get_the_archive_title() {
		if ( is_category() ) {
			$title = sprintf( __( 'Category: %s' ), single_cat_title( '', false ) );
		} elseif ( is_tag() ) {
			$title = sprintf( __( 'Tag: %s' ), single_tag_title( '', false ) );
		} elseif ( is_author() ) {
			$title = sprintf( __( 'Author: %s' ), '<span class="vcard">' . get_the_author() . '</span>' );
		} elseif ( is_year() ) {
			$title = sprintf( __( 'Year: %s' ), get_the_date( _x( 'Y', 'yearly archives date format' ) ) );
		} elseif ( is_month() ) {
			$title = sprintf( __( 'Month: %s' ), get_the_date( _x( 'F Y', 'monthly archives date format' ) ) );
		} elseif ( is_day() ) {
			$title = sprintf( __( 'Day: %s' ), get_the_date( _x( 'F j, Y', 'daily archives date format' ) ) );
		} elseif ( is_tax( 'post_format' ) ) {
			if ( is_tax( 'post_format', 'post-format-aside' ) ) {
				$title = _x( 'Asides', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-gallery' ) ) {
				$title = _x( 'Galleries', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-image' ) ) {
				$title = _x( 'Images', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-video' ) ) {
				$title = _x( 'Videos', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-quote' ) ) {
				$title = _x( 'Quotes', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-link' ) ) {
				$title = _x( 'Links', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-status' ) ) {
				$title = _x( 'Statuses', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-audio' ) ) {
				$title = _x( 'Audio', 'post format archive title' );
			} elseif ( is_tax( 'post_format', 'post-format-chat' ) ) {
				$title = _x( 'Chats', 'post format archive title' );
			}
		} elseif ( is_post_type_archive() ) {
			$title = sprintf( __( 'Archives: %s' ), post_type_archive_title( '', false ) );
		} elseif ( is_tax() ) {
			$tax = get_taxonomy( get_queried_object()->taxonomy );
			/* translators: 1: Taxonomy singular name, 2: Current taxonomy term */
			$title = sprintf( __( '%1$s: %2$s' ), $tax->labels->singular_name, single_term_title( '', false ) );
		} else {
			$title = __( 'Archives' );
		}

		/**
		 * Filter the archive title.
		 *
		 * @param string $title Archive title to be displayed.
		 *
		 * @since 4.1.0
		 *
		 */

		return apply_filters( 'get_the_archive_title', $title );
	}
}

add_action( 'wp_footer', 'eb_socialbuttonsinit' );
function eb_socialbuttonsinit() {
	if ( is_home() || is_archive() || ( is_single() && ( 'post' === get_post_type() ) ) ) {?>
		<script>!function (d, s, id) {
				var js, fjs = d.getElementsByTagName(s)[0];
				if (!d.getElementById(id)) {
					js = d.createElement(s);
					js.id = id;
					js.src = "//platform.twitter.com/widgets.js";
					fjs.parentNode.insertBefore(js, fjs);
				}
			}(document, "script", "twitter-wjs");</script>
		<div id="fb-root"></div>
		<script>(function (d, s, id) {
				var js, fjs = d.getElementsByTagName(s)[0];
				if (d.getElementById(id)) return;
				js = d.createElement(s);
				js.id = id;
				js.src = "//connect.facebook.net/en_US/sdk.js#xfbml=1&version=v2.4";
				fjs.parentNode.insertBefore(js, fjs);
			}(document, 'script', 'facebook-jssdk'));</script>
	<?php }
}
